#!/usr/bin/env python -Wd
import sys
import warnings
from os.path import abspath, dirname

import dj_database_url
import django
from django.conf import settings
from django.core.management import execute_from_command_line

# python -Wd, or run via coverage:
warnings.simplefilter("always", DeprecationWarning)

# Give feedback on used versions
sys.stderr.write(f"Using Python version {sys.version[:5]} from {sys.executable}\n")
sys.stderr.write(
    "Using Django version {} from {}\n".format(
        django.get_version(), dirname(abspath(django.__file__))
    )
)

if not settings.configured:
    settings.configure(
        DEBUG=False,
        DATABASES={
            "default": dj_database_url.config(env="PRIMARY_DATABASE", default="sqlite://:memory:"),
            "secondary": dj_database_url.config(
                env="SECONDARY_DATABASE", default="sqlite://:memory:"
            ),
        },
        DEFAULT_AUTO_FIELD="django.db.models.AutoField",
        TEST_RUNNER="django.test.runner.DiscoverRunner",
        INSTALLED_APPS=(
            "django.contrib.auth",
            "django.contrib.contenttypes",
            "django.contrib.messages",
            "django.contrib.sessions",
            "django.contrib.sites",
            "django.contrib.admin",
            "polymorphic",
            "polymorphic.tests",
        ),
        MIDDLEWARE=(
            "django.middleware.common.CommonMiddleware",
            "django.contrib.sessions.middleware.SessionMiddleware",
            "django.middleware.csrf.CsrfViewMiddleware",
            "django.contrib.auth.middleware.AuthenticationMiddleware",
            "django.contrib.messages.middleware.MessageMiddleware",
        ),
        SITE_ID=3,
        TEMPLATES=[
            {
                "BACKEND": "django.template.backends.django.DjangoTemplates",
                "DIRS": (),
                "OPTIONS": {
                    "loaders": (
                        "django.template.loaders.filesystem.Loader",
                        "django.template.loaders.app_directories.Loader",
                    ),
                    "context_processors": (
                        "django.template.context_processors.debug",
                        "django.template.context_processors.i18n",
                        "django.template.context_processors.media",
                        "django.template.context_processors.request",
                        "django.template.context_processors.static",
                        "django.contrib.messages.context_processors.messages",
                        "django.contrib.auth.context_processors.auth",
                    ),
                },
            }
        ],
        POLYMORPHIC_TEST_SWAPPABLE="polymorphic.swappedmodel",
        ROOT_URLCONF=None,
        SECRET_KEY="supersecret",
    )


DEFAULT_TEST_APPS = ["polymorphic"]


def runtests():
    other_args = list(filter(lambda arg: arg.startswith("-"), sys.argv[1:]))
    test_apps = (
        list(filter(lambda arg: not arg.startswith("-"), sys.argv[1:])) or DEFAULT_TEST_APPS
    )
    argv = sys.argv[:1] + ["test", "--traceback"] + other_args + test_apps
    execute_from_command_line(argv)


if __name__ == "__main__":
    runtests()



import uuid

import django
from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models.query import QuerySet

from polymorphic.managers import PolymorphicManager
from polymorphic.models import PolymorphicModel
from polymorphic.query import PolymorphicQuerySet
from polymorphic.showfields import ShowFieldContent, ShowFieldType, ShowFieldTypeAndContent


class PlainA(models.Model):
    field1 = models.CharField(max_length=10)


class PlainB(PlainA):
    field2 = models.CharField(max_length=10)


class PlainC(PlainB):
    field3 = models.CharField(max_length=10)


class Model2A(ShowFieldType, PolymorphicModel):
    field1 = models.CharField(max_length=10)
    polymorphic_showfield_deferred = True


class Model2B(Model2A):
    field2 = models.CharField(max_length=10)


class Model2C(Model2B):
    field3 = models.CharField(max_length=10)


class Model2D(Model2C):
    field4 = models.CharField(max_length=10)


class ModelExtraA(ShowFieldTypeAndContent, PolymorphicModel):
    field1 = models.CharField(max_length=10)


class ModelExtraB(ModelExtraA):
    field2 = models.CharField(max_length=10)


class ModelExtraC(ModelExtraB):
    field3 = models.CharField(max_length=10)


class ModelExtraExternal(models.Model):
    topic = models.CharField(max_length=10)


class ModelShow1(ShowFieldType, PolymorphicModel):
    field1 = models.CharField(max_length=10)
    m2m = models.ManyToManyField("self")


class ModelShow2(ShowFieldContent, PolymorphicModel):
    field1 = models.CharField(max_length=10)
    m2m = models.ManyToManyField("self")


class ModelShow3(ShowFieldTypeAndContent, PolymorphicModel):
    field1 = models.CharField(max_length=10)
    m2m = models.ManyToManyField("self")


class ModelShow1_plain(PolymorphicModel):
    field1 = models.CharField(max_length=10)


class ModelShow2_plain(ModelShow1_plain):
    field2 = models.CharField(max_length=10)


class Base(ShowFieldType, PolymorphicModel):
    polymorphic_showfield_deferred = True
    field_b = models.CharField(max_length=10)


class ModelX(Base):
    field_x = models.CharField(max_length=10)


class ModelY(Base):
    field_y = models.CharField(max_length=10)


class Enhance_Plain(models.Model):
    field_p = models.CharField(max_length=10)


class Enhance_Base(ShowFieldTypeAndContent, PolymorphicModel):
    base_id = models.AutoField(primary_key=True)
    field_b = models.CharField(max_length=10)


class Enhance_Inherit(Enhance_Base, Enhance_Plain):
    field_i = models.CharField(max_length=10)


class RelationBase(ShowFieldTypeAndContent, PolymorphicModel):
    field_base = models.CharField(max_length=10)
    fk = models.ForeignKey(
        "self", on_delete=models.CASCADE, null=True, related_name="relationbase_set"
    )
    m2m = models.ManyToManyField("self")


class RelationA(RelationBase):
    field_a = models.CharField(max_length=10)


class RelationB(RelationBase):
    field_b = models.CharField(max_length=10)


class RelationBC(RelationB):
    field_c = models.CharField(max_length=10)


class RelatingModel(models.Model):
    many2many = models.ManyToManyField(Model2A)


class One2OneRelatingModel(PolymorphicModel):
    one2one = models.OneToOneField(Model2A, on_delete=models.CASCADE)
    field1 = models.CharField(max_length=10)


class One2OneRelatingModelDerived(One2OneRelatingModel):
    field2 = models.CharField(max_length=10)


class ModelUnderRelParent(PolymorphicModel):
    field1 = models.CharField(max_length=10)
    _private = models.CharField(max_length=10)


class ModelUnderRelChild(PolymorphicModel):
    parent = models.ForeignKey(
        ModelUnderRelParent, on_delete=models.CASCADE, related_name="children"
    )
    _private2 = models.CharField(max_length=10)


class MyManagerQuerySet(PolymorphicQuerySet):
    def my_queryset_foo(self):
        # Just a method to prove the existence of the custom queryset.
        return self.all()


class MyManager(PolymorphicManager):
    queryset_class = MyManagerQuerySet

    def get_queryset(self):
        return super().get_queryset().order_by("-field1")

    def my_queryset_foo(self):
        return self.all().my_queryset_foo()


class ModelWithMyManager(ShowFieldTypeAndContent, Model2A):
    objects = MyManager()
    field4 = models.CharField(max_length=10)


class ModelWithMyManagerNoDefault(ShowFieldTypeAndContent, Model2A):
    objects = PolymorphicManager()
    my_objects = MyManager()
    field4 = models.CharField(max_length=10)


class ModelWithMyManagerDefault(ShowFieldTypeAndContent, Model2A):
    my_objects = MyManager()
    objects = PolymorphicManager()
    field4 = models.CharField(max_length=10)


class ModelWithMyManager2(ShowFieldTypeAndContent, Model2A):
    objects = MyManagerQuerySet.as_manager()
    field4 = models.CharField(max_length=10)


class MROBase1(ShowFieldType, PolymorphicModel):
    objects = MyManager()
    field1 = models.CharField(max_length=10)  # needed as MyManager uses it


class MROBase2(MROBase1):
    pass
    # No manager_inheritance_from_future or Meta set. test that polymorphic restores that.


class MROBase3(models.Model):
    # make sure 'id' field doesn't clash, detected by Django 1.11
    base_3_id = models.AutoField(primary_key=True)
    objects = models.Manager()


class MRODerived(MROBase2, MROBase3):
    if django.VERSION < (3, 0):

        class Meta:
            manager_inheritance_from_future = True


class ParentModelWithManager(PolymorphicModel):
    pass


class ChildModelWithManager(PolymorphicModel):
    # Also test whether foreign keys receive the manager:
    field1 = models.CharField(max_length=10)  # needed as MyManager uses it
    fk = models.ForeignKey(
        ParentModelWithManager, on_delete=models.CASCADE, related_name="childmodel_set"
    )
    objects = MyManager()


class PlainMyManagerQuerySet(QuerySet):
    def my_queryset_foo(self):
        # Just a method to prove the existence of the custom queryset.
        return self.all()


class PlainMyManager(models.Manager):
    def my_queryset_foo(self):
        return self.get_queryset().my_queryset_foo()

    def get_queryset(self):
        return PlainMyManagerQuerySet(self.model, using=self._db)


class PlainParentModelWithManager(models.Model):
    pass


class PlainChildModelWithManager(models.Model):
    fk = models.ForeignKey(
        PlainParentModelWithManager,
        on_delete=models.CASCADE,
        related_name="childmodel_set",
    )
    objects = PlainMyManager()


class BlogBase(ShowFieldTypeAndContent, PolymorphicModel):
    name = models.CharField(max_length=10)


class BlogA(BlogBase):
    info = models.CharField(max_length=10)


class BlogB(BlogBase):
    pass


class BlogEntry(ShowFieldTypeAndContent, PolymorphicModel):
    blog = models.ForeignKey(BlogA, on_delete=models.CASCADE)
    text = models.CharField(max_length=10)


class BlogEntry_limit_choices_to(ShowFieldTypeAndContent, PolymorphicModel):
    blog = models.ForeignKey(BlogBase, on_delete=models.CASCADE)
    text = models.CharField(max_length=10)


class ModelFieldNameTest(ShowFieldType, PolymorphicModel):
    modelfieldnametest = models.CharField(max_length=10)


class InitTestModel(ShowFieldType, PolymorphicModel):
    bar = models.CharField(max_length=100)

    def __init__(self, *args, **kwargs):
        kwargs["bar"] = self.x()
        super().__init__(*args, **kwargs)


class InitTestModelSubclass(InitTestModel):
    def x(self):
        return "XYZ"


# models from github issue


class Top(PolymorphicModel):
    name = models.CharField(max_length=50)


class Middle(Top):
    description = models.TextField()


class Bottom(Middle):
    author = models.CharField(max_length=50)


class UUIDProject(ShowFieldTypeAndContent, PolymorphicModel):
    uuid_primary_key = models.UUIDField(primary_key=True, default=uuid.uuid1)
    topic = models.CharField(max_length=30)


class UUIDArtProject(UUIDProject):
    artist = models.CharField(max_length=30)


class UUIDResearchProject(UUIDProject):
    supervisor = models.CharField(max_length=30)


class UUIDPlainA(models.Model):
    uuid_primary_key = models.UUIDField(primary_key=True, default=uuid.uuid1)
    field1 = models.CharField(max_length=10)


class UUIDPlainB(UUIDPlainA):
    field2 = models.CharField(max_length=10)


class UUIDPlainC(UUIDPlainB):
    field3 = models.CharField(max_length=10)


# base -> proxy


class ProxyBase(PolymorphicModel):
    some_data = models.CharField(max_length=128)


class ProxyChild(ProxyBase):
    class Meta:
        proxy = True


class NonProxyChild(ProxyBase):
    name = models.CharField(max_length=10)


# base -> proxy -> real models


class ProxiedBase(ShowFieldTypeAndContent, PolymorphicModel):
    name = models.CharField(max_length=10)


class ProxyModelBase(ProxiedBase):
    class Meta:
        proxy = True


class ProxyModelA(ProxyModelBase):
    field1 = models.CharField(max_length=10)


class ProxyModelB(ProxyModelBase):
    field2 = models.CharField(max_length=10)


# test bad field name
# class TestBadFieldModel(ShowFieldType, PolymorphicModel):
#    instance_of = models.CharField(max_length=10)

# validation error: "polymorphic.relatednameclash: Accessor for field 'polymorphic_ctype' clashes
# with related field 'ContentType.relatednameclash_set'." (reported by Andrew Ingram)
# fixed with related_name
class RelatedNameClash(ShowFieldType, PolymorphicModel):
    ctype = models.ForeignKey(ContentType, on_delete=models.CASCADE, null=True, editable=False)


# class with a parent_link to superclass, and a related_name back to subclass


class TestParentLinkAndRelatedName(ModelShow1_plain):
    superclass = models.OneToOneField(
        ModelShow1_plain,
        on_delete=models.CASCADE,
        parent_link=True,
        related_name="related_name_subclass",
    )


class CustomPkBase(ShowFieldTypeAndContent, PolymorphicModel):
    b = models.CharField(max_length=1)


class CustomPkInherit(CustomPkBase):
    custom_id = models.AutoField(primary_key=True)
    i = models.CharField(max_length=1)


class DateModel(PolymorphicModel):

    date = models.DateTimeField()


# Define abstract and swappable (being swapped for SwappedModel) models
# To test manager validation (should be skipped for such models)
class AbstractModel(PolymorphicModel):
    class Meta:
        abstract = True


class SwappableModel(AbstractModel):
    class Meta:
        swappable = "POLYMORPHIC_TEST_SWAPPABLE"


class SwappedModel(AbstractModel):
    pass


class InlineParent(models.Model):
    title = models.CharField(max_length=10)


class InlineModelA(PolymorphicModel):
    parent = models.ForeignKey(
        InlineParent, related_name="inline_children", on_delete=models.CASCADE
    )
    field1 = models.CharField(max_length=10)


class InlineModelB(InlineModelA):
    field2 = models.CharField(max_length=10)


class AbstractProject(PolymorphicModel):
    topic = models.CharField(max_length=30)

    class Meta:
        abstract = True


class ArtProject(AbstractProject):
    artist = models.CharField(max_length=30)


class Duck(PolymorphicModel):
    name = models.CharField(max_length=30)


class RedheadDuck(Duck):
    class Meta:
        proxy = True


class RubberDuck(Duck):
    class Meta:
        proxy = True


class MultiTableBase(PolymorphicModel):
    field1 = models.CharField(max_length=10)


class MultiTableDerived(MultiTableBase):
    field2 = models.CharField(max_length=10)


class SubclassSelectorAbstractBaseModel(PolymorphicModel):
    base_field = models.CharField(max_length=10, default="test_bf")


class SubclassSelectorAbstractModel(SubclassSelectorAbstractBaseModel):
    abstract_field = models.CharField(max_length=10, default="test_af")

    class Meta:
        abstract = True


class SubclassSelectorAbstractConcreteModel(SubclassSelectorAbstractModel):
    concrete_field = models.CharField(max_length=10, default="test_cf")


class SubclassSelectorProxyBaseModel(PolymorphicModel):
    base_field = models.CharField(max_length=10, default="test_bf")


class SubclassSelectorProxyModel(SubclassSelectorProxyBaseModel):
    class Meta:
        proxy = True


class SubclassSelectorProxyConcreteModel(SubclassSelectorProxyModel):
    concrete_field = models.CharField(max_length=10, default="test_cf")


class NonPolymorphicParent(PolymorphicModel, Group):
    test = models.CharField(max_length=22, default="test_non_polymorphic_parent")



"""
Seamless Polymorphic Inheritance for Django Models

Copyright:
This code and affiliated files are (C) by Bert Constantin and individual contributors.
Please see LICENSE and AUTHORS for more information.
"""

import pkg_resources

try:
    __version__ = pkg_resources.require("django-polymorphic")[0].version
except pkg_resources.DistributionNotFound:
    __version__ = None  # for RTD among others



import django
from django.test import TestCase

from polymorphic.tests.models import Bottom, Middle, Top

transform_arg = {"transform": repr} if django.VERSION >= (3, 2) else {}


class RegressionTests(TestCase):
    def test_for_query_result_incomplete_with_inheritance(self):
        """https://github.com/bconstantin/django_polymorphic/issues/15"""

        top = Top()
        top.save()
        middle = Middle()
        middle.save()
        bottom = Bottom()
        bottom.save()

        expected_queryset = [top, middle, bottom]
        self.assertQuerysetEqual(
            Top.objects.order_by("pk"), [repr(r) for r in expected_queryset], **transform_arg
        )

        expected_queryset = [middle, bottom]
        self.assertQuerysetEqual(
            Middle.objects.order_by("pk"), [repr(r) for r in expected_queryset], **transform_arg
        )

        expected_queryset = [bottom]
        self.assertQuerysetEqual(
            Bottom.objects.order_by("pk"), [repr(r) for r in expected_queryset], **transform_arg
        )



from django.contrib import admin
from django.urls import include, path, reverse_lazy
from django.views.generic import RedirectView

admin.autodiscover()

urlpatterns = [
    path("admin/", admin.site.urls),
    path("", RedirectView.as_view(url=reverse_lazy("admin:index"), permanent=False)),
]



from django.db import migrations, models

import polymorphic.showfields


class Migration(migrations.Migration):

    dependencies = [("contenttypes", "0002_remove_content_type_name")]

    operations = [
        migrations.CreateModel(
            name="NormalModelA",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name="Project",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("topic", models.CharField(max_length=30)),
            ],
            options={"abstract": False},
            bases=(polymorphic.showfields.ShowFieldContent, models.Model),
        ),
        migrations.CreateModel(
            name="ProxyBase",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("title", models.CharField(max_length=200)),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        related_name="polymorphic_pexp.proxybase_set+",
                        editable=False,
                        on_delete=models.CASCADE,
                        to="contenttypes.ContentType",
                        null=True,
                    ),
                ),
            ],
            options={"ordering": ("title",)},
        ),
        migrations.CreateModel(
            name="TestModelA",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
            options={"abstract": False},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, models.Model),
        ),
        migrations.CreateModel(
            name="UUIDModelA",
            fields=[
                (
                    "uuid_primary_key",
                    models.UUIDField(serialize=False, primary_key=True),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
            options={"abstract": False},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, models.Model),
        ),
        migrations.CreateModel(
            name="ArtProject",
            fields=[
                (
                    "project_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        on_delete=models.CASCADE,
                        to="pexp.Project",
                    ),
                ),
                ("artist", models.CharField(max_length=30)),
            ],
            options={"abstract": False},
            bases=("pexp.project",),
        ),
        migrations.CreateModel(
            name="NormalModelB",
            fields=[
                (
                    "normalmodela_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        on_delete=models.CASCADE,
                        to="pexp.NormalModelA",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            bases=("pexp.normalmodela",),
        ),
        migrations.CreateModel(
            name="ResearchProject",
            fields=[
                (
                    "project_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        on_delete=models.CASCADE,
                        to="pexp.Project",
                    ),
                ),
                ("supervisor", models.CharField(max_length=30)),
            ],
            options={"abstract": False},
            bases=("pexp.project",),
        ),
        migrations.CreateModel(
            name="TestModelB",
            fields=[
                (
                    "testmodela_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        on_delete=models.CASCADE,
                        to="pexp.TestModelA",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            options={"abstract": False},
            bases=("pexp.testmodela",),
        ),
        migrations.CreateModel(
            name="UUIDModelB",
            fields=[
                (
                    "uuidmodela_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        on_delete=models.CASCADE,
                        to="pexp.UUIDModelA",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            options={"abstract": False},
            bases=("pexp.uuidmodela",),
        ),
        migrations.AddField(
            model_name="uuidmodela",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                related_name="polymorphic_pexp.uuidmodela_set+",
                editable=False,
                on_delete=models.CASCADE,
                to="contenttypes.ContentType",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="testmodela",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                related_name="polymorphic_pexp.testmodela_set+",
                editable=False,
                on_delete=models.CASCADE,
                to="contenttypes.ContentType",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                related_name="polymorphic_pexp.project_set+",
                editable=False,
                on_delete=models.CASCADE,
                to="contenttypes.ContentType",
                null=True,
            ),
        ),
        migrations.CreateModel(
            name="ProxyA", fields=[], options={"proxy": True}, bases=("pexp.proxybase",)
        ),
        migrations.CreateModel(
            name="ProxyB", fields=[], options={"proxy": True}, bases=("pexp.proxybase",)
        ),
        migrations.CreateModel(
            name="NormalModelC",
            fields=[
                (
                    "normalmodelb_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        on_delete=models.CASCADE,
                        to="pexp.NormalModelB",
                    ),
                ),
                ("field3", models.CharField(max_length=10)),
            ],
            bases=("pexp.normalmodelb",),
        ),
        migrations.CreateModel(
            name="TestModelC",
            fields=[
                (
                    "testmodelb_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        on_delete=models.CASCADE,
                        to="pexp.TestModelB",
                    ),
                ),
                ("field3", models.CharField(max_length=10)),
                (
                    "field4",
                    models.ManyToManyField(related_name="related_c", to="pexp.TestModelB"),
                ),
            ],
            options={"abstract": False},
            bases=("pexp.testmodelb",),
        ),
        migrations.CreateModel(
            name="UUIDModelC",
            fields=[
                (
                    "uuidmodelb_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        on_delete=models.CASCADE,
                        to="pexp.UUIDModelB",
                    ),
                ),
                ("field3", models.CharField(max_length=10)),
            ],
            options={"abstract": False},
            bases=("pexp.uuidmodelb",),
        ),
    ]



from django.db import models
from django.utils.dates import MONTHS_3
from django.utils.translation import gettext_lazy as _

from polymorphic.models import PolymorphicModel


class Order(models.Model):
    """
    An example order that has polymorphic relations
    """

    title = models.CharField(_("Title"), max_length=200)

    class Meta:
        verbose_name = _("Organisation")
        verbose_name_plural = _("Organisations")
        ordering = ("title",)

    def __str__(self):
        return self.title


class Payment(PolymorphicModel):
    """
    A generic payment model.
    """

    order = models.ForeignKey(Order, on_delete=models.CASCADE)
    currency = models.CharField(default="USD", max_length=3)
    amount = models.DecimalField(max_digits=10, decimal_places=2)

    class Meta:
        verbose_name = _("Payment")
        verbose_name_plural = _("Payments")

    def __str__(self):
        return f"{self.currency} {self.amount}"


class CreditCardPayment(Payment):
    """
    Credit card
    """

    MONTH_CHOICES = [(i, n) for i, n in sorted(MONTHS_3.items())]

    card_type = models.CharField(max_length=10)
    expiry_month = models.PositiveSmallIntegerField(choices=MONTH_CHOICES)
    expiry_year = models.PositiveIntegerField()

    class Meta:
        verbose_name = _("Credit Card Payment")
        verbose_name_plural = _("Credit Card Payments")


class BankPayment(Payment):
    """
    Payment by bank
    """

    bank_name = models.CharField(max_length=100)
    swift = models.CharField(max_length=20)

    class Meta:
        verbose_name = _("Bank Payment")
        verbose_name_plural = _("Bank Payments")


class SepaPayment(Payment):
    """
    Payment by SEPA (EU)
    """

    iban = models.CharField(max_length=34)
    bic = models.CharField(max_length=11)

    class Meta:
        verbose_name = _("SEPA Payment")
        verbose_name_plural = _("SEPA Payments")



"""
This module is a scratchpad for general development, testing & debugging
"""

from django.core.management import BaseCommand
from pexp.models import *


class Command(BaseCommand):
    help = ""

    def handle_noargs(self, **options):
        Project.objects.all().delete()
        o = Project.objects.create(topic="John's gathering")
        o = ArtProject.objects.create(topic="Sculpting with Tim", artist="T. Turner")
        o = ResearchProject.objects.create(topic="Swallow Aerodynamics", supervisor="Dr. Winter")
        print(Project.objects.all())



"""
Internal utils
"""
import django


def add_media(dest, media):
    """
    Optimized version of django.forms.Media.__add__() that doesn't create new objects.
    """
    if django.VERSION >= (2, 2):
        dest._css_lists.extend(media._css_lists)
        dest._js_lists.extend(media._js_lists)
    elif django.VERSION >= (2, 0):
        combined = dest + media
        dest._css = combined._css
        dest._js = combined._js
    else:
        dest.add_css(media._css)
        dest.add_js(media._js)



import os

DEBUG = True

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS
PROJECT_ROOT = os.path.dirname(os.path.dirname(__file__))

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(PROJECT_ROOT, "example.db"),
    }
}

SITE_ID = 1

# Make this unique, and don't share it with anybody.
SECRET_KEY = "5$f%)&amp;a4tc*bg(79+ku!7o$kri-duw99@hq_)va^_kaw9*l)!7"


# Language
# TIME_ZONE = 'America/Chicago'
LANGUAGE_CODE = "en-us"
USE_I18N = True
USE_L10N = True
USE_TZ = True

# Paths
MEDIA_ROOT = ""
MEDIA_URL = "/media/"
STATIC_ROOT = ""
STATIC_URL = "/static/"

# Apps
STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
)

MIDDLEWARE = (
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
)

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": (),
        "OPTIONS": {
            "loaders": (
                "django.template.loaders.filesystem.Loader",
                "django.template.loaders.app_directories.Loader",
            ),
            "context_processors": (
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.request",
                "django.template.context_processors.static",
                "django.contrib.messages.context_processors.messages",
                "django.contrib.auth.context_processors.auth",
            ),
        },
    }
]

ROOT_URLCONF = "example.urls"

WSGI_APPLICATION = "example.wsgi.application"

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.admin",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "polymorphic",  # needed if you want to use the polymorphic admin
    "pexp",  # this Django app is for testing and experimentation; not needed otherwise
    "orders",
)

TEST_RUNNER = "django.test.runner.DiscoverRunner"  # silence system checks

# Logging configuration
LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "filters": {"require_debug_false": {"()": "django.utils.log.RequireDebugFalse"}},
    "handlers": {
        "mail_admins": {
            "level": "ERROR",
            "filters": ["require_debug_false"],
            "class": "django.utils.log.AdminEmailHandler",
        }
    },
    "loggers": {
        "django.request": {
            "handlers": ["mail_admins"],
            "level": "ERROR",
            "propagate": True,
        }
    },
}



from django.contrib import admin
from django.contrib.contenttypes.models import ContentType
from django.utils.html import escape

from polymorphic.admin import (
    PolymorphicChildModelAdmin,
    PolymorphicChildModelFilter,
    PolymorphicInlineSupportMixin,
    PolymorphicParentModelAdmin,
    StackedPolymorphicInline,
)
from polymorphic.tests.admintestcase import AdminTestCase
from polymorphic.tests.models import (
    InlineModelA,
    InlineModelB,
    InlineParent,
    Model2A,
    Model2B,
    Model2C,
    Model2D,
)


class PolymorphicAdminTests(AdminTestCase):
    def test_admin_registration(self):
        """
        Test how the registration works
        """

        @self.register(Model2A)
        class Model2Admin(PolymorphicParentModelAdmin):
            base_model = Model2A
            list_filter = (PolymorphicChildModelFilter,)
            child_models = (Model2B, Model2C, Model2D)

        @self.register(Model2B)
        @self.register(Model2C)
        @self.register(Model2D)
        class Model2ChildAdmin(PolymorphicChildModelAdmin):
            base_model = Model2A
            base_fieldsets = (("Base fields", {"fields": ("field1",)}),)

        # -- add page
        ct_id = ContentType.objects.get_for_model(Model2D).pk
        self.admin_get_add(Model2A)  # shows type page
        self.admin_get_add(Model2A, qs=f"?ct_id={ct_id}")  # shows type page

        self.admin_get_add(Model2A)  # shows type page
        self.admin_post_add(
            Model2A,
            {"field1": "A", "field2": "B", "field3": "C", "field4": "D"},
            qs=f"?ct_id={ct_id}",
        )

        d_obj = Model2A.objects.all()[0]
        self.assertEqual(d_obj.__class__, Model2D)
        self.assertEqual(d_obj.field1, "A")
        self.assertEqual(d_obj.field2, "B")

        # -- list page
        self.admin_get_changelist(Model2A)  # asserts 200

        # -- edit
        response = self.admin_get_change(Model2A, d_obj.pk)
        self.assertContains(response, "field4")
        self.admin_post_change(
            Model2A,
            d_obj.pk,
            {"field1": "A2", "field2": "B2", "field3": "C2", "field4": "D2"},
        )

        d_obj.refresh_from_db()
        self.assertEqual(d_obj.field1, "A2")
        self.assertEqual(d_obj.field2, "B2")
        self.assertEqual(d_obj.field3, "C2")
        self.assertEqual(d_obj.field4, "D2")

        # -- history
        self.admin_get_history(Model2A, d_obj.pk)

        # -- delete
        self.admin_get_delete(Model2A, d_obj.pk)
        self.admin_post_delete(Model2A, d_obj.pk)
        self.assertRaises(Model2A.DoesNotExist, lambda: d_obj.refresh_from_db())

    def test_admin_inlines(self):
        """
        Test the registration of inline models.
        """

        class InlineModelAChild(StackedPolymorphicInline.Child):
            model = InlineModelA

        class InlineModelBChild(StackedPolymorphicInline.Child):
            model = InlineModelB

        class Inline(StackedPolymorphicInline):
            model = InlineModelA
            child_inlines = (InlineModelAChild, InlineModelBChild)

        @self.register(InlineParent)
        class InlineParentAdmin(PolymorphicInlineSupportMixin, admin.ModelAdmin):
            inlines = (Inline,)

        parent = InlineParent.objects.create(title="FOO")
        self.assertEqual(parent.inline_children.count(), 0)

        # -- get edit page
        response = self.admin_get_change(InlineParent, parent.pk)

        # Make sure the fieldset has the right data exposed in data-inline-formset
        self.assertContains(response, "childTypes")
        self.assertContains(response, escape('"type": "inlinemodela"'))
        self.assertContains(response, escape('"type": "inlinemodelb"'))

        # -- post edit page
        self.admin_post_change(
            InlineParent,
            parent.pk,
            {
                "title": "FOO2",
                "inline_children-INITIAL_FORMS": 0,
                "inline_children-TOTAL_FORMS": 1,
                "inline_children-MIN_NUM_FORMS": 0,
                "inline_children-MAX_NUM_FORMS": 1000,
                "inline_children-0-parent": parent.pk,
                "inline_children-0-polymorphic_ctype": ContentType.objects.get_for_model(
                    InlineModelB
                ).pk,
                "inline_children-0-field1": "A2",
                "inline_children-0-field2": "B2",
            },
        )

        parent.refresh_from_db()
        self.assertEqual(parent.title, "FOO2")
        self.assertEqual(parent.inline_children.count(), 1)
        child = parent.inline_children.all()[0]
        self.assertEqual(child.__class__, InlineModelB)
        self.assertEqual(child.field1, "A2")
        self.assertEqual(child.field2, "B2")



import json

from django.template import Library
from django.utils.encoding import force_str
from django.utils.text import capfirst
from django.utils.translation import gettext

from polymorphic.formsets import BasePolymorphicModelFormSet

register = Library()


@register.filter()
def include_empty_form(formset):
    """
    Make sure the "empty form" is included when displaying a formset (typically table with input rows)
    """
    yield from formset

    if hasattr(formset, "empty_forms"):
        # BasePolymorphicModelFormSet
        yield from formset.empty_forms
    else:
        # Standard Django formset
        yield formset.empty_form


@register.filter
def as_script_options(formset):
    """
    A JavaScript data structure for the JavaScript code

    This generates the ``data-options`` attribute for ``jquery.django-inlines.js``
    The formset may define the following extra attributes:

    - ``verbose_name``
    - ``add_text``
    - ``show_add_button``
    """
    verbose_name = getattr(formset, "verbose_name", formset.model._meta.verbose_name)
    options = {
        "prefix": formset.prefix,
        "pkFieldName": formset.model._meta.pk.name,
        "addText": getattr(formset, "add_text", None)
        or gettext("Add another %(verbose_name)s") % {"verbose_name": capfirst(verbose_name)},
        "showAddButton": getattr(formset, "show_add_button", True),
        "deleteText": gettext("Delete"),
    }

    if isinstance(formset, BasePolymorphicModelFormSet):
        # Allow to add different types
        options["childTypes"] = [
            {
                "name": force_str(model._meta.verbose_name),
                "type": model._meta.model_name,
            }
            for model in formset.child_forms.keys()
        ]

    return json.dumps(options)


@register.filter
def as_form_type(form):
    """
    Usage: ``{{ form|as_form_type }}``
    """
    return form._meta.model._meta.model_name


@register.filter
def as_model_name(model):
    """
    Usage: ``{{ model|as_model_name }}``
    """
    return model._meta.model_name



from django.template import Library, Node, TemplateSyntaxError

register = Library()


class BreadcrumbScope(Node):
    def __init__(self, base_opts, nodelist):
        self.base_opts = base_opts
        self.nodelist = nodelist  # Note, takes advantage of Node.child_nodelists

    @classmethod
    def parse(cls, parser, token):
        bits = token.split_contents()
        if len(bits) == 2:
            (tagname, base_opts) = bits
            base_opts = parser.compile_filter(base_opts)
            nodelist = parser.parse(("endbreadcrumb_scope",))
            parser.delete_first_token()

            return cls(base_opts=base_opts, nodelist=nodelist)
        else:
            raise TemplateSyntaxError(f"{token.contents[0]} tag expects 1 argument")

    def render(self, context):
        # app_label is really hard to overwrite in the standard Django ModelAdmin.
        # To insert it in the template, the entire render_change_form() and delete_view() have to copied and adjusted.
        # Instead, have an assignment tag that inserts that in the template.
        base_opts = self.base_opts.resolve(context)
        new_vars = {}
        if base_opts and not isinstance(base_opts, str):
            new_vars = {
                "app_label": base_opts.app_label,  # What this is all about
                "opts": base_opts,
            }

        new_scope = context.push()
        new_scope.update(new_vars)
        html = self.nodelist.render(context)
        context.pop()
        return html


@register.tag
def breadcrumb_scope(parser, token):
    """
    Easily allow the breadcrumb to be generated in the admin change templates.
    """
    return BreadcrumbScope.parse(parser, token)



"""
Django Admin support for polymorphic inlines.

Each row in the inline can correspond with a different subclass.
"""
from functools import partial

from django.conf import settings
from django.contrib.admin.options import InlineModelAdmin
from django.contrib.admin.utils import flatten_fieldsets
from django.core.exceptions import ImproperlyConfigured
from django.forms import Media

from polymorphic.formsets import (
    BasePolymorphicInlineFormSet,
    PolymorphicFormSetChild,
    UnsupportedChildType,
    polymorphic_child_forms_factory,
)
from polymorphic.formsets.utils import add_media

from .helpers import PolymorphicInlineSupportMixin


class PolymorphicInlineModelAdmin(InlineModelAdmin):
    """
    A polymorphic inline, where each formset row can be a different form.

    Note that:

    * Permissions are only checked on the base model.
    * The child inlines can't override the base model fields, only this parent inline can do that.
    """

    formset = BasePolymorphicInlineFormSet

    #: The extra media to add for the polymorphic inlines effect.
    #: This can be redefined for subclasses.
    polymorphic_media = Media(
        js=(
            "admin/js/vendor/jquery/jquery{}.js".format("" if settings.DEBUG else ".min"),
            "admin/js/jquery.init.js",
            "polymorphic/js/polymorphic_inlines.js",
        ),
        css={"all": ("polymorphic/css/polymorphic_inlines.css",)},
    )

    #: The extra forms to show
    #: By default there are no 'extra' forms as the desired type is unknown.
    #: Instead, add each new item using JavaScript that first offers a type-selection.
    extra = 0

    #: Inlines for all model sub types that can be displayed in this inline.
    #: Each row is a :class:`PolymorphicInlineModelAdmin.Child`
    child_inlines = ()

    def __init__(self, parent_model, admin_site):
        super().__init__(parent_model, admin_site)

        # Extra check to avoid confusion
        # While we could monkeypatch the admin here, better stay explicit.
        parent_admin = admin_site._registry.get(parent_model, None)
        if parent_admin is not None:  # Can be None during check
            if not isinstance(parent_admin, PolymorphicInlineSupportMixin):
                raise ImproperlyConfigured(
                    "To use polymorphic inlines, add the `PolymorphicInlineSupportMixin` mixin "
                    "to the ModelAdmin that hosts the inline."
                )

        # While the inline is created per request, the 'request' object is not known here.
        # Hence, creating all child inlines unconditionally, without checking permissions.
        self.child_inline_instances = self.get_child_inline_instances()

        # Create a lookup table
        self._child_inlines_lookup = {}
        for child_inline in self.child_inline_instances:
            self._child_inlines_lookup[child_inline.model] = child_inline

    def get_child_inline_instances(self):
        """
        :rtype List[PolymorphicInlineModelAdmin.Child]
        """
        instances = []
        for ChildInlineType in self.child_inlines:
            instances.append(ChildInlineType(parent_inline=self))
        return instances

    def get_child_inline_instance(self, model):
        """
        Find the child inline for a given model.

        :rtype: PolymorphicInlineModelAdmin.Child
        """
        try:
            return self._child_inlines_lookup[model]
        except KeyError:
            raise UnsupportedChildType(f"Model '{model.__name__}' not found in child_inlines")

    def get_formset(self, request, obj=None, **kwargs):
        """
        Construct the inline formset class.

        This passes all class attributes to the formset.

        :rtype: type
        """
        # Construct the FormSet class
        FormSet = super().get_formset(request, obj=obj, **kwargs)

        # Instead of completely redefining super().get_formset(), we use
        # the regular inlineformset_factory(), and amend that with our extra bits.
        # This code line is the essence of what polymorphic_inlineformset_factory() does.
        FormSet.child_forms = polymorphic_child_forms_factory(
            formset_children=self.get_formset_children(request, obj=obj)
        )
        return FormSet

    def get_formset_children(self, request, obj=None):
        """
        The formset 'children' provide the details for all child models that are part of this formset.
        It provides a stripped version of the modelform/formset factory methods.
        """
        formset_children = []
        for child_inline in self.child_inline_instances:
            # TODO: the children can be limited here per request based on permissions.
            formset_children.append(child_inline.get_formset_child(request, obj=obj))
        return formset_children

    def get_fieldsets(self, request, obj=None):
        """
        Hook for specifying fieldsets.
        """
        if self.fieldsets:
            return self.fieldsets
        else:
            return []  # Avoid exposing fields to the child

    def get_fields(self, request, obj=None):
        if self.fields:
            return self.fields
        else:
            return []  # Avoid exposing fields to the child

    @property
    def media(self):
        # The media of the inline focuses on the admin settings,
        # whether to expose the scripts for filter_horizontal etc..
        # The admin helper exposes the inline + formset media.
        base_media = super().media
        all_media = Media()
        add_media(all_media, base_media)

        # Add all media of the child inline instances
        for child_instance in self.child_inline_instances:
            child_media = child_instance.media

            # Avoid adding the same media object again and again
            if child_media._css != base_media._css and child_media._js != base_media._js:
                add_media(all_media, child_media)

        add_media(all_media, self.polymorphic_media)

        return all_media

    class Child(InlineModelAdmin):
        """
        The child inline; which allows configuring the admin options
        for the child appearance.

        Note that not all options will be honored by the parent, notably the formset options:
        * :attr:`extra`
        * :attr:`min_num`
        * :attr:`max_num`

        The model form options however, will all be read.
        """

        formset_child = PolymorphicFormSetChild
        extra = 0  # TODO: currently unused for the children.

        def __init__(self, parent_inline):
            self.parent_inline = parent_inline
            super(PolymorphicInlineModelAdmin.Child, self).__init__(
                parent_inline.parent_model, parent_inline.admin_site
            )

        def get_formset(self, request, obj=None, **kwargs):
            # The child inline is only used to construct the form,
            # and allow to override the form field attributes.
            # The formset is created by the parent inline.
            raise RuntimeError("The child get_formset() is not used.")

        def get_fields(self, request, obj=None):
            if self.fields:
                return self.fields

            # Standard Django logic, use the form to determine the fields.
            # The form needs to pass through all factory logic so all 'excludes' are set as well.
            # Default Django does: form = self.get_formset(request, obj, fields=None).form
            # Use 'fields=None' avoids recursion in the field autodetection.
            form = self.get_formset_child(request, obj, fields=None).get_form()
            return list(form.base_fields) + list(self.get_readonly_fields(request, obj))

        def get_formset_child(self, request, obj=None, **kwargs):
            """
            Return the formset child that the parent inline can use to represent us.

            :rtype: PolymorphicFormSetChild
            """
            # Similar to the normal get_formset(), the caller may pass fields to override the defaults settings
            # in the inline. In Django's GenericInlineModelAdmin.get_formset() this is also used in the same way,
            # to make sure the 'exclude' also contains the GFK fields.
            #
            # Hence this code is almost identical to InlineModelAdmin.get_formset()
            # and GenericInlineModelAdmin.get_formset()
            #
            # Transfer the local inline attributes to the formset child,
            # this allows overriding settings.
            if "fields" in kwargs:
                fields = kwargs.pop("fields")
            else:
                fields = flatten_fieldsets(self.get_fieldsets(request, obj))

            if self.exclude is None:
                exclude = []
            else:
                exclude = list(self.exclude)

            exclude.extend(self.get_readonly_fields(request, obj))
            # Add forcefully, as Django 1.10 doesn't include readonly fields.
            exclude.append("polymorphic_ctype")

            if self.exclude is None and hasattr(self.form, "_meta") and self.form._meta.exclude:
                # Take the custom ModelForm's Meta.exclude into account only if the
                # InlineModelAdmin doesn't define its own.
                exclude.extend(self.form._meta.exclude)

            # can_delete = self.can_delete and self.has_delete_permission(request, obj)
            defaults = {
                "form": self.form,
                "fields": fields,
                "exclude": exclude or None,
                "formfield_callback": partial(self.formfield_for_dbfield, request=request),
            }
            defaults.update(kwargs)

            # This goes through the same logic that get_formset() calls
            # by passing the inline class attributes to modelform_factory()
            FormSetChildClass = self.formset_child
            return FormSetChildClass(self.model, **defaults)


class StackedPolymorphicInline(PolymorphicInlineModelAdmin):
    """
    Stacked inline for django-polymorphic models.
    Since tabular doesn't make much sense with changed fields, just offer this one.
    """

    #: The default template to use.
    template = "admin/polymorphic/edit_inline/stacked.html"



from django.db import models

from polymorphic.models import PolymorphicModel
from polymorphic.showfields import ShowFieldContent, ShowFieldTypeAndContent


class Project(ShowFieldContent, PolymorphicModel):
    """Polymorphic model"""

    topic = models.CharField(max_length=30)


class ArtProject(Project):
    artist = models.CharField(max_length=30)


class ResearchProject(Project):
    supervisor = models.CharField(max_length=30)


class UUIDModelA(ShowFieldTypeAndContent, PolymorphicModel):
    """UUID as primary key example"""

    uuid_primary_key = models.UUIDField(primary_key=True)
    field1 = models.CharField(max_length=10)


class UUIDModelB(UUIDModelA):
    field2 = models.CharField(max_length=10)


class UUIDModelC(UUIDModelB):
    field3 = models.CharField(max_length=10)


class ProxyBase(PolymorphicModel):
    """Proxy model example - a single table with multiple types."""

    title = models.CharField(max_length=200)

    def __unicode__(self):
        return f"<ProxyBase[type={self.polymorphic_ctype}]: {self.title}>"

    class Meta:
        ordering = ("title",)


class ProxyA(ProxyBase):
    class Meta:
        proxy = True

    def __unicode__(self):
        return f"<ProxyA: {self.title}>"


class ProxyB(ProxyBase):
    class Meta:
        proxy = True

    def __unicode__(self):
        return f"<ProxyB: {self.title}>"


# Internals for management command tests


class TestModelA(ShowFieldTypeAndContent, PolymorphicModel):
    field1 = models.CharField(max_length=10)


class TestModelB(TestModelA):
    field2 = models.CharField(max_length=10)


class TestModelC(TestModelB):
    field3 = models.CharField(max_length=10)
    field4 = models.ManyToManyField(TestModelB, related_name="related_c")


class NormalModelA(models.Model):
    """Normal Django inheritance, no polymorphic behavior"""

    field1 = models.CharField(max_length=10)


class NormalModelB(NormalModelA):
    field2 = models.CharField(max_length=10)


class NormalModelC(NormalModelB):
    field3 = models.CharField(max_length=10)



"""
PolymorphicQuerySet support functions
"""
import copy
from collections import deque

from django.apps import apps
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import FieldDoesNotExist, FieldError
from django.db import models
from django.db.models import Q
from django.db.models.fields.related import ForeignObjectRel, RelatedField
from django.db.utils import DEFAULT_DB_ALIAS

# These functions implement the additional filter- and Q-object functionality.
# They form a kind of small framework for easily adding more
# functionality to filters and Q objects.
# Probably a more general queryset enhancement class could be made out of them.
from polymorphic import compat

###################################################################################
# PolymorphicQuerySet support functions


def translate_polymorphic_filter_definitions_in_kwargs(
    queryset_model, kwargs, using=DEFAULT_DB_ALIAS
):
    """
    Translate the keyword argument list for PolymorphicQuerySet.filter()

    Any kwargs with special polymorphic functionality are replaced in the kwargs
    dict with their vanilla django equivalents.

    For some kwargs a direct replacement is not possible, as a Q object is needed
    instead to implement the required functionality. In these cases the kwarg is
    deleted from the kwargs dict and a Q object is added to the return list.

    Modifies: kwargs dict
    Returns: a list of non-keyword-arguments (Q objects) to be added to the filter() query.
    """
    additional_args = []
    for field_path, val in kwargs.copy().items():  # Python 3 needs copy

        new_expr = _translate_polymorphic_filter_definition(
            queryset_model, field_path, val, using=using
        )

        if isinstance(new_expr, tuple):
            # replace kwargs element
            del kwargs[field_path]
            kwargs[new_expr[0]] = new_expr[1]

        elif isinstance(new_expr, models.Q):
            del kwargs[field_path]
            additional_args.append(new_expr)

    return additional_args


def translate_polymorphic_Q_object(queryset_model, potential_q_object, using=DEFAULT_DB_ALIAS):
    def tree_node_correct_field_specs(my_model, node):
        "process all children of this Q node"
        for i in range(len(node.children)):
            child = node.children[i]

            if isinstance(child, (tuple, list)):
                # this Q object child is a tuple => a kwarg like Q( instance_of=ModelB )
                key, val = child
                new_expr = _translate_polymorphic_filter_definition(
                    my_model, key, val, using=using
                )
                if new_expr:
                    node.children[i] = new_expr
            else:
                # this Q object child is another Q object, recursively process this as well
                tree_node_correct_field_specs(my_model, child)

    if isinstance(potential_q_object, models.Q):
        tree_node_correct_field_specs(queryset_model, potential_q_object)

    return potential_q_object


def translate_polymorphic_filter_definitions_in_args(queryset_model, args, using=DEFAULT_DB_ALIAS):
    """
    Translate the non-keyword argument list for PolymorphicQuerySet.filter()

    In the args list, we return all kwargs to Q-objects that contain special
    polymorphic functionality with their vanilla django equivalents.
    We traverse the Q object tree for this (which is simple).


    Returns: modified Q objects
    """
    return [
        translate_polymorphic_Q_object(queryset_model, copy.deepcopy(q), using=using) for q in args
    ]


def _translate_polymorphic_filter_definition(
    queryset_model, field_path, field_val, using=DEFAULT_DB_ALIAS
):
    """
    Translate a keyword argument (field_path=field_val), as used for
    PolymorphicQuerySet.filter()-like functions (and Q objects).

    A kwarg with special polymorphic functionality is translated into
    its vanilla django equivalent, which is returned, either as tuple
    (field_path, field_val) or as Q object.

    Returns: kwarg tuple or Q object or None (if no change is required)
    """

    # handle instance_of expressions or alternatively,
    # if this is a normal Django filter expression, return None
    if field_path == "instance_of":
        return create_instanceof_q(field_val, using=using)
    elif field_path == "not_instance_of":
        return create_instanceof_q(field_val, not_instance_of=True, using=using)
    elif "___" not in field_path:
        return None  # no change

    # filter expression contains '___' (i.e. filter for polymorphic field)
    # => get the model class specified in the filter expression
    newpath = translate_polymorphic_field_path(queryset_model, field_path)
    return (newpath, field_val)


def translate_polymorphic_field_path(queryset_model, field_path):
    """
    Translate a field path from a keyword argument, as used for
    PolymorphicQuerySet.filter()-like functions (and Q objects).
    Supports leading '-' (for order_by args).

    E.g.: if queryset_model is ModelA, then "ModelC___field3" is translated
    into modela__modelb__modelc__field3.
    Returns: translated path (unchanged, if no translation needed)
    """
    if not isinstance(field_path, str):
        raise ValueError(f"Expected field name as string: {field_path}")

    classname, sep, pure_field_path = field_path.partition("___")
    if not sep:
        return field_path
    assert classname, "PolymorphicModel: %s: bad field specification" % field_path

    negated = False
    if classname[0] == "-":
        negated = True
        classname = classname.lstrip("-")

    if "__" in classname:
        # the user has app label prepended to class name via __ => use Django's get_model function
        appname, sep, classname = classname.partition("__")
        model = apps.get_model(appname, classname)
        assert model, "PolymorphicModel: model {} (in app {}) not found!".format(
            model.__name__,
            appname,
        )
        if not issubclass(model, queryset_model):
            e = (
                'PolymorphicModel: queryset filter error: "'
                + model.__name__
                + '" is not derived from "'
                + queryset_model.__name__
                + '"'
            )
            raise AssertionError(e)

    else:
        # the user has only given us the class name via ___
        # => select the model from the sub models of the queryset base model

        # Test whether it's actually a regular relation__ _fieldname (the field starting with an _)
        # so no tripple ClassName___field was intended.
        try:
            # This also retreives M2M relations now (including reverse foreign key relations)
            field = queryset_model._meta.get_field(classname)

            if isinstance(field, (RelatedField, ForeignObjectRel)):
                # Can also test whether the field exists in the related object to avoid ambiguity between
                # class names and field names, but that never happens when your class names are in CamelCase.
                return field_path  # No exception raised, field does exist.
        except FieldDoesNotExist:
            pass

        submodels = _get_all_sub_models(queryset_model)
        model = submodels.get(classname, None)
        assert model, "PolymorphicModel: model {} not found (not a subclass of {})!".format(
            classname,
            queryset_model.__name__,
        )

    basepath = _create_base_path(queryset_model, model)

    if negated:
        newpath = "-"
    else:
        newpath = ""

    newpath += basepath
    if basepath:
        newpath += "__"

    newpath += pure_field_path
    return newpath


def _get_all_sub_models(base_model):
    """#Collect all sub-models, this should be optimized (cached)"""
    result = {}
    queue = deque([base_model])

    while queue:
        model = queue.popleft()
        if issubclass(model, models.Model) and model != models.Model:
            # model name is occurring twice in submodel inheritance tree => Error
            if model.__name__ in result and model != result[model.__name__]:
                raise FieldError(
                    "PolymorphicModel: model name alone is ambiguous: %s.%s and %s.%s match!\n"
                    "In this case, please use the syntax: applabel__ModelName___field"
                    % (
                        model._meta.app_label,
                        model.__name__,
                        result[model.__name__]._meta.app_label,
                        result[model.__name__].__name__,
                    )
                )

            result[model.__name__] = model
        queue.extend(model.__subclasses__())

    return result


def _create_base_path(baseclass, myclass):
    # create new field path for expressions, e.g. for baseclass=ModelA, myclass=ModelC
    # 'modelb__modelc" is returned
    for b in myclass.__bases__:
        if b == baseclass:
            return _get_query_related_name(myclass)

        path = _create_base_path(baseclass, b)
        if path:
            if b._meta.abstract or b._meta.proxy:
                return _get_query_related_name(myclass)
            else:
                return path + "__" + _get_query_related_name(myclass)
    return ""


def _get_query_related_name(myclass):
    for f in myclass._meta.local_fields:
        if isinstance(f, models.OneToOneField) and f.remote_field.parent_link:
            return f.related_query_name()

    # Fallback to undetected name,
    # this happens on proxy models (e.g. SubclassSelectorProxyModel)
    return myclass.__name__.lower()


def create_instanceof_q(modellist, not_instance_of=False, using=DEFAULT_DB_ALIAS):
    """
    Helper function for instance_of / not_instance_of
    Creates and returns a Q object that filters for the models in modellist,
    including all subclasses of these models (as we want to do the same
    as pythons isinstance() ).
    .
    We recursively collect all __subclasses__(), create a Q filter for each,
    and or-combine these Q objects. This could be done much more
    efficiently however (regarding the resulting sql), should an optimization
    be needed.
    """
    if not modellist:
        return None

    if not isinstance(modellist, (list, tuple)):
        from .models import PolymorphicModel

        if issubclass(modellist, PolymorphicModel):
            modellist = [modellist]
        else:
            raise TypeError(
                "PolymorphicModel: instance_of expects a list of (polymorphic) "
                "models or a single (polymorphic) model"
            )

    contenttype_ids = _get_mro_content_type_ids(modellist, using)
    q = Q(polymorphic_ctype__in=sorted(contenttype_ids))
    if not_instance_of:
        q = ~q
    return q


def _get_mro_content_type_ids(models, using):
    contenttype_ids = set()
    for model in models:
        ct = ContentType.objects.db_manager(using).get_for_model(model, for_concrete_model=False)
        contenttype_ids.add(ct.pk)
        subclasses = model.__subclasses__()
        if subclasses:
            contenttype_ids.update(_get_mro_content_type_ids(subclasses, using))
    return contenttype_ids



from django.contrib.contenttypes.forms import (
    BaseGenericInlineFormSet,
    generic_inlineformset_factory,
)
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.forms.models import ModelForm

from .models import (
    BasePolymorphicModelFormSet,
    PolymorphicFormSetChild,
    polymorphic_child_forms_factory,
)


class GenericPolymorphicFormSetChild(PolymorphicFormSetChild):
    """
    Formset child for generic inlines
    """

    def __init__(self, *args, **kwargs):
        self.ct_field = kwargs.pop("ct_field", "content_type")
        self.fk_field = kwargs.pop("fk_field", "object_id")
        super().__init__(*args, **kwargs)

    def get_form(self, ct_field="content_type", fk_field="object_id", **kwargs):
        """
        Construct the form class for the formset child.
        """
        exclude = list(self.exclude)
        extra_exclude = kwargs.pop("extra_exclude", None)
        if extra_exclude:
            exclude += list(extra_exclude)

        # Make sure the GFK fields are excluded by default
        # This is similar to what generic_inlineformset_factory() does
        # if there is no field called `ct_field` let the exception propagate
        opts = self.model._meta
        ct_field = opts.get_field(self.ct_field)

        if (
            not isinstance(ct_field, models.ForeignKey)
            or ct_field.remote_field.model != ContentType
        ):
            raise Exception("fk_name '%s' is not a ForeignKey to ContentType" % ct_field)

        fk_field = opts.get_field(self.fk_field)  # let the exception propagate
        exclude.extend([ct_field.name, fk_field.name])
        kwargs["exclude"] = exclude

        return super().get_form(**kwargs)


class BaseGenericPolymorphicInlineFormSet(BaseGenericInlineFormSet, BasePolymorphicModelFormSet):
    """
    Polymorphic formset variation for inline generic formsets
    """


def generic_polymorphic_inlineformset_factory(
    model,
    formset_children,
    form=ModelForm,
    formset=BaseGenericPolymorphicInlineFormSet,
    ct_field="content_type",
    fk_field="object_id",
    # Base form
    # TODO: should these fields be removed in favor of creating
    # the base form as a formset child too?
    fields=None,
    exclude=None,
    extra=1,
    can_order=False,
    can_delete=True,
    max_num=None,
    formfield_callback=None,
    validate_max=False,
    for_concrete_model=True,
    min_num=None,
    validate_min=False,
    child_form_kwargs=None,
):
    """
    Construct the class for a generic inline polymorphic formset.

    All arguments are identical to :func:`~django.contrib.contenttypes.forms.generic_inlineformset_factory`,
    with the exception of the ``formset_children`` argument.

    :param formset_children: A list of all child :class:`PolymorphicFormSetChild` objects
                             that tell the inline how to render the child model types.
    :type formset_children: Iterable[PolymorphicFormSetChild]
    :rtype: type
    """
    kwargs = {
        "model": model,
        "form": form,
        "formfield_callback": formfield_callback,
        "formset": formset,
        "ct_field": ct_field,
        "fk_field": fk_field,
        "extra": extra,
        "can_delete": can_delete,
        "can_order": can_order,
        "fields": fields,
        "exclude": exclude,
        "min_num": min_num,
        "max_num": max_num,
        "validate_min": validate_min,
        "validate_max": validate_max,
        "for_concrete_model": for_concrete_model,
        # 'localized_fields': localized_fields,
        # 'labels': labels,
        # 'help_texts': help_texts,
        # 'error_messages': error_messages,
        # 'field_classes': field_classes,
    }
    if child_form_kwargs is None:
        child_form_kwargs = {}

    child_kwargs = {
        # 'exclude': exclude,
        "ct_field": ct_field,
        "fk_field": fk_field,
    }
    if child_form_kwargs:
        child_kwargs.update(child_form_kwargs)

    FormSet = generic_inlineformset_factory(**kwargs)
    FormSet.child_forms = polymorphic_child_forms_factory(formset_children, **child_kwargs)
    return FormSet



from django.contrib import admin
from pexp.models import *

from polymorphic.admin import (
    PolymorphicChildModelAdmin,
    PolymorphicChildModelFilter,
    PolymorphicParentModelAdmin,
)


class ProjectAdmin(PolymorphicParentModelAdmin):
    base_model = Project  # Can be set explicitly.
    list_filter = (PolymorphicChildModelFilter,)
    child_models = (Project, ArtProject, ResearchProject)


class ProjectChildAdmin(PolymorphicChildModelAdmin):
    base_model = Project  # Can be set explicitly.

    # On purpose, only have the shared fields here.
    # The fields of the derived model should still be displayed.
    base_fieldsets = (("Base fields", {"fields": ("topic",)}),)


admin.site.register(Project, ProjectAdmin)
admin.site.register(ArtProject, ProjectChildAdmin)
admin.site.register(ResearchProject, ProjectChildAdmin)


class UUIDModelAAdmin(PolymorphicParentModelAdmin):
    list_filter = (PolymorphicChildModelFilter,)
    child_models = (UUIDModelA, UUIDModelB)


class UUIDModelAChildAdmin(PolymorphicChildModelAdmin):
    pass


admin.site.register(UUIDModelA, UUIDModelAAdmin)
admin.site.register(UUIDModelB, UUIDModelAChildAdmin)
admin.site.register(UUIDModelC, UUIDModelAChildAdmin)


class ProxyAdmin(PolymorphicParentModelAdmin):
    list_filter = (PolymorphicChildModelFilter,)
    child_models = (ProxyA, ProxyB)


class ProxyChildAdmin(PolymorphicChildModelAdmin):
    pass


admin.site.register(ProxyBase, ProxyAdmin)
admin.site.register(ProxyA, ProxyChildAdmin)
admin.site.register(ProxyB, ProxyChildAdmin)



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("contenttypes", "0002_remove_content_type_name")]

    operations = [
        migrations.CreateModel(
            name="Order",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("title", models.CharField(max_length=200, verbose_name="Title")),
            ],
            options={
                "ordering": ("title",),
                "verbose_name": "Organisation",
                "verbose_name_plural": "Organisations",
            },
        ),
        migrations.CreateModel(
            name="Payment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("currency", models.CharField(default=b"USD", max_length=3)),
                ("amount", models.DecimalField(max_digits=10, decimal_places=2)),
            ],
            options={"verbose_name": "Payment", "verbose_name_plural": "Payments"},
        ),
        migrations.CreateModel(
            name="BankPayment",
            fields=[
                (
                    "payment_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        on_delete=models.CASCADE,
                        to="orders.Payment",
                    ),
                ),
                ("bank_name", models.CharField(max_length=100)),
                ("swift", models.CharField(max_length=20)),
            ],
            options={
                "verbose_name": "Bank Payment",
                "verbose_name_plural": "Bank Payments",
            },
            bases=("orders.payment",),
        ),
        migrations.CreateModel(
            name="CreditCardPayment",
            fields=[
                (
                    "payment_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        on_delete=models.CASCADE,
                        to="orders.Payment",
                    ),
                ),
                ("card_type", models.CharField(max_length=10)),
                (
                    "expiry_month",
                    models.PositiveSmallIntegerField(
                        choices=[
                            (1, "jan"),
                            (2, "feb"),
                            (3, "mar"),
                            (4, "apr"),
                            (5, "may"),
                            (6, "jun"),
                            (7, "jul"),
                            (8, "aug"),
                            (9, "sep"),
                            (10, "oct"),
                            (11, "nov"),
                            (12, "dec"),
                        ]
                    ),
                ),
                ("expiry_year", models.PositiveIntegerField()),
            ],
            options={
                "verbose_name": "Credit Card Payment",
                "verbose_name_plural": "Credit Card Payments",
            },
            bases=("orders.payment",),
        ),
        migrations.CreateModel(
            name="SepaPayment",
            fields=[
                (
                    "payment_ptr",
                    models.OneToOneField(
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        on_delete=models.CASCADE,
                        to="orders.Payment",
                    ),
                ),
                ("iban", models.CharField(max_length=34)),
                ("bic", models.CharField(max_length=11)),
            ],
            options={
                "verbose_name": "Bank Payment",
                "verbose_name_plural": "Bank Payments",
            },
            bases=("orders.payment",),
        ),
        migrations.AddField(
            model_name="payment",
            name="order",
            field=models.ForeignKey(to="orders.Order", on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name="payment",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                related_name="polymorphic_orders.payment_set+",
                editable=False,
                on_delete=models.CASCADE,
                to="contenttypes.ContentType",
                null=True,
            ),
        ),
    ]



"""
This allows creating formsets where each row can be a different form type.
The logic of the formsets work similar to the standard Django formsets;
there are factory methods to construct the classes with the proper form settings.

The "parent" formset hosts the entire model and their child model.
For every child type, there is an :class:`PolymorphicFormSetChild` instance
that describes how to display and construct the child.
It's parameters are very similar to the parent's factory method.
"""
from .generic import (  # Can import generic here, as polymorphic already depends on the 'contenttypes' app.
    BaseGenericPolymorphicInlineFormSet,
    GenericPolymorphicFormSetChild,
    generic_polymorphic_inlineformset_factory,
)
from .models import (
    BasePolymorphicInlineFormSet,
    BasePolymorphicModelFormSet,
    PolymorphicFormSetChild,
    UnsupportedChildType,
    polymorphic_child_forms_factory,
    polymorphic_inlineformset_factory,
    polymorphic_modelformset_factory,
)

__all__ = (
    "BasePolymorphicModelFormSet",
    "BasePolymorphicInlineFormSet",
    "PolymorphicFormSetChild",
    "UnsupportedChildType",
    "polymorphic_modelformset_factory",
    "polymorphic_inlineformset_factory",
    "polymorphic_child_forms_factory",
    "BaseGenericPolymorphicInlineFormSet",
    "GenericPolymorphicFormSetChild",
    "generic_polymorphic_inlineformset_factory",
)



"""
This module is a scratchpad for general development, testing & debugging
Well, even more so than pcmd.py. You best ignore p2cmd.py.
"""
import sys
import time
from pprint import pprint
from random import Random

from django.core.management import BaseCommand
from django.db import connection
from pexp.models import *

rnd = Random()


def show_queries():
    print()
    print("QUERIES:", len(connection.queries))
    pprint(connection.queries)
    print()
    connection.queries = []


def print_timing(func, message="", iterations=1):
    def wrapper(*arg):
        results = []
        connection.queries_log.clear()
        for i in range(iterations):
            t1 = time.time()
            x = func(*arg)
            t2 = time.time()
            results.append((t2 - t1) * 1000.0)
        res_sum = 0
        for r in results:
            res_sum += r
        print(
            "%s%-19s: %.4f ms, %i queries (%i times)"
            % (message, func.func_name, res_sum, len(connection.queries), iterations)
        )
        sys.stdout.flush()

    return wrapper


class Command(BaseCommand):
    help = ""

    def handle_noargs(self, **options):
        if False:
            TestModelA.objects.all().delete()
            a = TestModelA.objects.create(field1="A1")
            b = TestModelB.objects.create(field1="B1", field2="B2")
            c = TestModelC.objects.create(field1="C1", field2="C2", field3="C3")
            connection.queries_log.clear()
            print(TestModelC.base_objects.all())
            show_queries()

        if False:
            TestModelA.objects.all().delete()
            for i in range(1000):
                a = TestModelA.objects.create(field1=str(i % 100))
                b = TestModelB.objects.create(field1=str(i % 100), field2=str(i % 200))
                c = TestModelC.objects.create(
                    field1=str(i % 100), field2=str(i % 200), field3=str(i % 300)
                )
                if i % 100 == 0:
                    print(i)

        f = print_timing(poly_sql_query, iterations=1000)
        f()

        f = print_timing(poly_sql_query2, iterations=1000)
        f()

        return

        NormalModelA.objects.all().delete()
        a = NormalModelA.objects.create(field1="A1")
        b = NormalModelB.objects.create(field1="B1", field2="B2")
        c = NormalModelC.objects.create(field1="C1", field2="C2", field3="C3")
        qs = TestModelA.objects.raw("SELECT * from pexp_testmodela")
        for o in list(qs):
            print(o)


def poly_sql_query():
    cursor = connection.cursor()
    cursor.execute(
        """
        SELECT id, pexp_testmodela.field1, pexp_testmodelb.field2, pexp_testmodelc.field3
        FROM pexp_testmodela
        LEFT OUTER JOIN pexp_testmodelb
        ON pexp_testmodela.id = pexp_testmodelb.testmodela_ptr_id
        LEFT OUTER JOIN pexp_testmodelc
        ON pexp_testmodelb.testmodela_ptr_id = pexp_testmodelc.testmodelb_ptr_id
        WHERE pexp_testmodela.field1=%i
        ORDER BY pexp_testmodela.id
        """
        % rnd.randint(0, 100)
    )
    # row=cursor.fetchone()
    return


def poly_sql_query2():
    cursor = connection.cursor()
    cursor.execute(
        """
        SELECT id, pexp_testmodela.field1
        FROM pexp_testmodela
        WHERE pexp_testmodela.field1=%i
        ORDER BY pexp_testmodela.id
        """
        % rnd.randint(0, 100)
    )
    # row=cursor.fetchone()
    return



# Settings file to allow parsing API documentation of Django modules,
# and provide defaults to use in the documentation.
#
# This file is placed in a subdirectory,
# so the docs root won't be detected by find_packages()

# Display sane URLs in the docs:
STATIC_URL = "/static/"

# Avoid error for missing the secret key
SECRET_KEY = "docs"

INSTALLED_APPS = ["django.contrib.contenttypes"]



"""
This module is a scratchpad for general development, testing & debugging
"""

import sys
import time
from pprint import pprint

from django.core.management import BaseCommand
from django.db import connection
from pexp.models import *

num_objects = 1000


def show_queries():
    print()
    print("QUERIES:", len(connection.queries))
    pprint(connection.queries)
    print()
    connection.queries_log.clear()


###################################################################################
# benchmark wrappers


def print_timing(func, message="", iterations=1):
    def wrapper(*arg):
        results = []
        connection.queries_log.clear()
        for i in range(iterations):
            t1 = time.time()
            x = func(*arg)
            t2 = time.time()
            results.append((t2 - t1) * 1000.0)
        res_sum = 0
        for r in results:
            res_sum += r
        median = res_sum / len(results)
        print(
            "%s%-19s: %.0f ms, %i queries"
            % (message, func.func_name, median, len(connection.queries) / len(results))
        )
        sys.stdout.flush()

    return wrapper


def run_vanilla_any_poly(func, iterations=1):
    f = print_timing(func, "     ", iterations)
    f(NormalModelC)
    f = print_timing(func, "poly ", iterations)
    f(TestModelC)


###################################################################################
# benchmarks


def bench_create(model):
    for i in range(num_objects):
        model.objects.create(
            field1="abc" + str(i), field2="abcd" + str(i), field3="abcde" + str(i)
        )
    # print 'count:',model.objects.count()


def bench_load1(model):
    for o in model.objects.all():
        pass


def bench_load1_short(model):
    for i in range(num_objects / 100):
        for o in model.objects.all()[:100]:
            pass


def bench_load2(model):
    for o in model.objects.all():
        f1 = o.field1
        f2 = o.field2
        f3 = o.field3


def bench_load2_short(model):
    for i in range(num_objects / 100):
        for o in model.objects.all()[:100]:
            f1 = o.field1
            f2 = o.field2
            f3 = o.field3


def bench_delete(model):
    model.objects.all().delete()


###################################################################################
# Command


class Command(BaseCommand):
    help = ""

    def handle_noargs(self, **options):
        func_list = [
            (bench_delete, 1),
            (bench_create, 1),
            (bench_load1, 5),
            (bench_load1_short, 5),
            (bench_load2, 5),
            (bench_load2_short, 5),
        ]
        for f, iterations in func_list:
            run_vanilla_any_poly(f, iterations=iterations)



import uuid

import django.db.models.deletion
import django.db.models.manager
from django.db import migrations, models

import polymorphic.showfields


class Migration(migrations.Migration):

    initial = True

    dependencies = [("contenttypes", "0002_remove_content_type_name")]

    operations = [
        migrations.CreateModel(
            name="SwappableModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.swappablemodel_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"swappable": "POLYMORPHIC_TEST_SWAPPABLE"},
        ),
        migrations.CreateModel(
            name="Base",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field_b", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldType, models.Model),
        ),
        migrations.CreateModel(
            name="BlogBase",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, models.Model),
        ),
        migrations.CreateModel(
            name="BlogEntry",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.CharField(max_length=10)),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.blogentry_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, models.Model),
        ),
        migrations.CreateModel(
            name="BlogEntry_limit_choices_to",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, models.Model),
        ),
        migrations.CreateModel(
            name="ChildModelWithManager",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                )
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="CustomPkBase",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("b", models.CharField(max_length=1)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, models.Model),
        ),
        migrations.CreateModel(
            name="DateModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateTimeField()),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.datemodel_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="Enhance_Base",
            fields=[
                ("base_id", models.AutoField(primary_key=True, serialize=False)),
                ("field_b", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, models.Model),
        ),
        migrations.CreateModel(
            name="Enhance_Plain",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field_p", models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name="InitTestModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("bar", models.CharField(max_length=100)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldType, models.Model),
        ),
        migrations.CreateModel(
            name="MgrInheritA",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
            managers=[("mgrA", django.db.models.manager.Manager())],
        ),
        migrations.CreateModel(
            name="Model2A",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldType, models.Model),
        ),
        migrations.CreateModel(
            name="ModelExtraA",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, models.Model),
        ),
        migrations.CreateModel(
            name="ModelExtraExternal",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("topic", models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name="ModelFieldNameTest",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("modelfieldnametest", models.CharField(max_length=10)),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.modelfieldnametest_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldType, models.Model),
        ),
        migrations.CreateModel(
            name="ModelShow1",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
                (
                    "m2m",
                    models.ManyToManyField(
                        related_name="_modelshow1_m2m_+", to="tests.ModelShow1"
                    ),
                ),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.modelshow1_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldType, models.Model),
        ),
        migrations.CreateModel(
            name="ModelShow1_plain",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="ModelShow2",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
                (
                    "m2m",
                    models.ManyToManyField(
                        related_name="_modelshow2_m2m_+", to="tests.ModelShow2"
                    ),
                ),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.modelshow2_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldContent, models.Model),
        ),
        migrations.CreateModel(
            name="ModelShow3",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
                (
                    "m2m",
                    models.ManyToManyField(
                        related_name="_modelshow3_m2m_+", to="tests.ModelShow3"
                    ),
                ),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.modelshow3_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, models.Model),
        ),
        migrations.CreateModel(
            name="ModelUnderRelChild",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("_private2", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="ModelUnderRelParent",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
                ("_private", models.CharField(max_length=10)),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.modelunderrelparent_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="MROBase1",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldType, models.Model),
        ),
        migrations.CreateModel(
            name="MROBase3",
            fields=[("base_3_id", models.AutoField(primary_key=True, serialize=False))],
        ),
        migrations.CreateModel(
            name="One2OneRelatingModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="ParentModelWithManager",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.parentmodelwithmanager_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="PlainA",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name="PlainChildModelWithManager",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                )
            ],
        ),
        migrations.CreateModel(
            name="PlainParentModelWithManager",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                )
            ],
        ),
        migrations.CreateModel(
            name="ProxiedBase",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=10)),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.proxiedbase_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, models.Model),
        ),
        migrations.CreateModel(
            name="ProxyBase",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("some_data", models.CharField(max_length=128)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="RelatedNameClash",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.ContentType",
                    ),
                ),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.relatednameclash_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldType, models.Model),
        ),
        migrations.CreateModel(
            name="RelatingModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                )
            ],
        ),
        migrations.CreateModel(
            name="RelationBase",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field_base", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, models.Model),
        ),
        migrations.CreateModel(
            name="SwappedModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.swappedmodel_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False},
        ),
        migrations.CreateModel(
            name="Top",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=50)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="UUIDPlainA",
            fields=[
                (
                    "uuid_primary_key",
                    models.UUIDField(default=uuid.uuid1, primary_key=True, serialize=False),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name="UUIDProject",
            fields=[
                (
                    "uuid_primary_key",
                    models.UUIDField(default=uuid.uuid1, primary_key=True, serialize=False),
                ),
                ("topic", models.CharField(max_length=30)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, models.Model),
        ),
        migrations.CreateModel(
            name="BlogA",
            fields=[
                (
                    "blogbase_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.BlogBase",
                    ),
                ),
                ("info", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.blogbase",),
        ),
        migrations.CreateModel(
            name="BlogB",
            fields=[
                (
                    "blogbase_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.BlogBase",
                    ),
                )
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.blogbase",),
        ),
        migrations.CreateModel(
            name="CustomPkInherit",
            fields=[
                (
                    "custompkbase_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        to="tests.CustomPkBase",
                    ),
                ),
                ("custom_id", models.AutoField(primary_key=True, serialize=False)),
                ("i", models.CharField(max_length=1)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.custompkbase",),
        ),
        migrations.CreateModel(
            name="Enhance_Inherit",
            fields=[
                (
                    "enhance_plain_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        to="tests.Enhance_Plain",
                    ),
                ),
                (
                    "enhance_base_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.Enhance_Base",
                    ),
                ),
                ("field_i", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.enhance_base", "tests.enhance_plain"),
        ),
        migrations.CreateModel(
            name="InitTestModelSubclass",
            fields=[
                (
                    "inittestmodel_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.InitTestModel",
                    ),
                )
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.inittestmodel",),
        ),
        migrations.CreateModel(
            name="MgrInheritB",
            fields=[
                (
                    "mgrinherita_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.MgrInheritA",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            bases=("tests.mgrinherita",),
            managers=[("mgrB", django.db.models.manager.Manager())],
        ),
        migrations.CreateModel(
            name="Middle",
            fields=[
                (
                    "top_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.Top",
                    ),
                ),
                ("description", models.TextField()),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.top",),
        ),
        migrations.CreateModel(
            name="Model2B",
            fields=[
                (
                    "model2a_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.Model2A",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.model2a",),
        ),
        migrations.CreateModel(
            name="ModelExtraB",
            fields=[
                (
                    "modelextraa_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.ModelExtraA",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.modelextraa",),
        ),
        migrations.CreateModel(
            name="ModelShow2_plain",
            fields=[
                (
                    "modelshow1_plain_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.ModelShow1_plain",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.modelshow1_plain",),
        ),
        migrations.CreateModel(
            name="ModelWithMyManager",
            fields=[
                (
                    "model2a_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.Model2A",
                    ),
                ),
                ("field4", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, "tests.model2a"),
        ),
        migrations.CreateModel(
            name="ModelWithMyManager2",
            fields=[
                (
                    "model2a_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.Model2A",
                    ),
                ),
                ("field4", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, "tests.model2a"),
        ),
        migrations.CreateModel(
            name="ModelWithMyManagerDefault",
            fields=[
                (
                    "model2a_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.Model2A",
                    ),
                ),
                ("field4", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, "tests.model2a"),
            managers=[
                ("my_objects", django.db.models.manager.Manager()),
                ("objects", django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name="ModelWithMyManagerNoDefault",
            fields=[
                (
                    "model2a_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.Model2A",
                    ),
                ),
                ("field4", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, "tests.model2a"),
        ),
        migrations.CreateModel(
            name="ModelX",
            fields=[
                (
                    "base_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.Base",
                    ),
                ),
                ("field_x", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.base",),
        ),
        migrations.CreateModel(
            name="ModelY",
            fields=[
                (
                    "base_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.Base",
                    ),
                ),
                ("field_y", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.base",),
        ),
        migrations.CreateModel(
            name="MROBase2",
            fields=[
                (
                    "mrobase1_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.MROBase1",
                    ),
                )
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.mrobase1",),
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("base_objects", django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name="NonProxyChild",
            fields=[
                (
                    "proxybase_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.ProxyBase",
                    ),
                ),
                ("name", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.proxybase",),
        ),
        migrations.CreateModel(
            name="One2OneRelatingModelDerived",
            fields=[
                (
                    "one2onerelatingmodel_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.One2OneRelatingModel",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.one2onerelatingmodel",),
        ),
        migrations.CreateModel(
            name="PlainB",
            fields=[
                (
                    "plaina_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.PlainA",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            bases=("tests.plaina",),
        ),
        migrations.CreateModel(
            name="RelationA",
            fields=[
                (
                    "relationbase_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.RelationBase",
                    ),
                ),
                ("field_a", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.relationbase",),
        ),
        migrations.CreateModel(
            name="RelationB",
            fields=[
                (
                    "relationbase_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.RelationBase",
                    ),
                ),
                ("field_b", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.relationbase",),
        ),
        migrations.CreateModel(
            name="TestParentLinkAndRelatedName",
            fields=[
                (
                    "superclass",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="related_name_subclass",
                        serialize=False,
                        to="tests.ModelShow1_plain",
                    ),
                )
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.modelshow1_plain",),
        ),
        migrations.CreateModel(
            name="UUIDArtProject",
            fields=[
                (
                    "uuidproject_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.UUIDProject",
                    ),
                ),
                ("artist", models.CharField(max_length=30)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.uuidproject",),
        ),
        migrations.CreateModel(
            name="UUIDPlainB",
            fields=[
                (
                    "uuidplaina_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.UUIDPlainA",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            bases=("tests.uuidplaina",),
        ),
        migrations.CreateModel(
            name="UUIDResearchProject",
            fields=[
                (
                    "uuidproject_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.UUIDProject",
                    ),
                ),
                ("supervisor", models.CharField(max_length=30)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.uuidproject",),
        ),
        migrations.AddField(
            model_name="uuidproject",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.uuidproject_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="top",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.top_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="relationbase",
            name="fk",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="relationbase_set",
                to="tests.RelationBase",
            ),
        ),
        migrations.AddField(
            model_name="relationbase",
            name="m2m",
            field=models.ManyToManyField(
                related_name="_relationbase_m2m_+", to="tests.RelationBase"
            ),
        ),
        migrations.AddField(
            model_name="relationbase",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.relationbase_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="relatingmodel",
            name="many2many",
            field=models.ManyToManyField(to="tests.Model2A"),
        ),
        migrations.AddField(
            model_name="proxybase",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.proxybase_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="plainchildmodelwithmanager",
            name="fk",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="childmodel_set",
                to="tests.PlainParentModelWithManager",
            ),
        ),
        migrations.AddField(
            model_name="one2onerelatingmodel",
            name="one2one",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE, to="tests.Model2A"
            ),
        ),
        migrations.AddField(
            model_name="one2onerelatingmodel",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.one2onerelatingmodel_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="mrobase1",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.mrobase1_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="modelunderrelchild",
            name="parent",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="children",
                to="tests.ModelUnderRelParent",
            ),
        ),
        migrations.AddField(
            model_name="modelunderrelchild",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.modelunderrelchild_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="modelshow1_plain",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.modelshow1_plain_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="modelextraa",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.modelextraa_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="model2a",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.model2a_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="inittestmodel",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.inittestmodel_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="enhance_base",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.enhance_base_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="custompkbase",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.custompkbase_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="childmodelwithmanager",
            name="fk",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="childmodel_set",
                to="tests.ParentModelWithManager",
            ),
        ),
        migrations.AddField(
            model_name="childmodelwithmanager",
            name="field1",
            field=models.CharField(
                max_length=10,
            ),
        ),
        migrations.AddField(
            model_name="childmodelwithmanager",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.childmodelwithmanager_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="blogentry_limit_choices_to",
            name="blog",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="tests.BlogBase"
            ),
        ),
        migrations.AddField(
            model_name="blogentry_limit_choices_to",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.blogentry_limit_choices_to_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="blogbase",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.blogbase_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.AddField(
            model_name="base",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.base_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.CreateModel(
            name="ProxyChild",
            fields=[],
            options={"proxy": True},
            bases=("tests.proxybase",),
        ),
        migrations.CreateModel(
            name="ProxyModelBase",
            fields=[],
            options={"proxy": True},
            bases=("tests.proxiedbase",),
        ),
        migrations.CreateModel(
            name="Bottom",
            fields=[
                (
                    "middle_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.Middle",
                    ),
                ),
                ("author", models.CharField(max_length=50)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.middle",),
        ),
        migrations.CreateModel(
            name="MgrInheritC",
            fields=[
                (
                    "mgrinheritb_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.MgrInheritB",
                    ),
                )
            ],
            bases=(polymorphic.showfields.ShowFieldTypeAndContent, "tests.mgrinheritb"),
        ),
        migrations.CreateModel(
            name="Model2C",
            fields=[
                (
                    "model2b_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.Model2B",
                    ),
                ),
                ("field3", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.model2b",),
        ),
        migrations.CreateModel(
            name="ModelExtraC",
            fields=[
                (
                    "modelextrab_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.ModelExtraB",
                    ),
                ),
                ("field3", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.modelextrab",),
        ),
        migrations.CreateModel(
            name="MRODerived",
            fields=[
                (
                    "mrobase3_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        to="tests.MROBase3",
                    ),
                ),
                (
                    "mrobase2_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.MROBase2",
                    ),
                ),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.mrobase2", "tests.mrobase3"),
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("base_objects", django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name="PlainC",
            fields=[
                (
                    "plainb_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.PlainB",
                    ),
                ),
                ("field3", models.CharField(max_length=10)),
            ],
            bases=("tests.plainb",),
        ),
        migrations.CreateModel(
            name="ProxyModelA",
            fields=[
                (
                    "proxiedbase_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.ProxiedBase",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.proxymodelbase",),
        ),
        migrations.CreateModel(
            name="ProxyModelB",
            fields=[
                (
                    "proxiedbase_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.ProxiedBase",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.proxymodelbase",),
        ),
        migrations.CreateModel(
            name="RelationBC",
            fields=[
                (
                    "relationb_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.RelationB",
                    ),
                ),
                ("field_c", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.relationb",),
        ),
        migrations.CreateModel(
            name="UUIDPlainC",
            fields=[
                (
                    "uuidplainb_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.UUIDPlainB",
                    ),
                ),
                ("field3", models.CharField(max_length=10)),
            ],
            bases=("tests.uuidplainb",),
        ),
        migrations.AddField(
            model_name="blogentry",
            name="blog",
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="tests.BlogA"),
        ),
        migrations.CreateModel(
            name="Model2D",
            fields=[
                (
                    "model2c_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.Model2C",
                    ),
                ),
                ("field4", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.model2c",),
        ),
        migrations.CreateModel(
            name="InlineModelBase",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                )
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="InlineParent",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=10)),
            ],
        ),
        migrations.CreateModel(
            name="InlineModelA",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="InlineModelB",
            fields=[
                (
                    "inlinemodela_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.InlineModelA",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.inlinemodela",),
        ),
        migrations.AddField(
            model_name="inlinemodela",
            name="parent",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="tests.InlineParent",
                related_name="inline_children",
            ),
        ),
        migrations.AddField(
            model_name="inlinemodela",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.inlinemodela_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.CreateModel(
            name="ArtProject",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("topic", models.CharField(max_length=30)),
                ("artist", models.CharField(max_length=30)),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.artproject_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False},
        ),
        migrations.CreateModel(
            name="Duck",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=30)),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.duck_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False},
        ),
        migrations.CreateModel(
            name="MultiTableBase",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("field1", models.CharField(max_length=10)),
            ],
            options={"abstract": False},
        ),
        migrations.CreateModel(
            name="MultiTableDerived",
            fields=[
                (
                    "multitablebase_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.MultiTableBase",
                    ),
                ),
                ("field2", models.CharField(max_length=10)),
            ],
            options={"abstract": False},
            bases=("tests.multitablebase",),
        ),
        migrations.AddField(
            model_name="multitablebase",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.multitablebase_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.CreateModel(
            name="RedheadDuck",
            fields=[],
            options={"proxy": True},
            bases=("tests.duck",),
        ),
        migrations.CreateModel(
            name="RubberDuck", fields=[], options={"proxy": True}, bases=("tests.duck",)
        ),
        migrations.CreateModel(
            name="SubclassSelectorAbstractBaseModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("base_field", models.CharField(default="test_bf", max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="SubclassSelectorProxyBaseModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("base_field", models.CharField(default="test_bf", max_length=10)),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.subclassselectorproxybasemodel_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
        ),
        migrations.CreateModel(
            name="SubclassSelectorAbstractConcreteModel",
            fields=[
                (
                    "subclassselectorabstractbasemodel_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.SubclassSelectorAbstractBaseModel",
                    ),
                ),
                ("abstract_field", models.CharField(default="test_af", max_length=10)),
                ("concrete_field", models.CharField(default="test_cf", max_length=10)),
            ],
            options={"abstract": False},
            bases=("tests.subclassselectorabstractbasemodel",),
        ),
        migrations.AddField(
            model_name="subclassselectorabstractbasemodel",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_tests.subclassselectorabstractbasemodel_set+",
                to="contenttypes.ContentType",
            ),
        ),
        migrations.CreateModel(
            name="SubclassSelectorProxyModel",
            fields=[],
            options={"proxy": True, "indexes": []},
            bases=("tests.subclassselectorproxybasemodel",),
        ),
        migrations.CreateModel(
            name="SubclassSelectorProxyConcreteModel",
            fields=[
                (
                    "subclassselectorproxybasemodel_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.SubclassSelectorProxyBaseModel",
                    ),
                ),
                ("concrete_field", models.CharField(default="test_cf", max_length=10)),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("tests.subclassselectorproxymodel",),
        ),
        migrations.CreateModel(
            name="NonPolymorphicParent",
            fields=[
                (
                    "group_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="auth.Group",
                    ),
                ),
                (
                    "polymorphic_ctype",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="polymorphic_tests.nonpolymorphicparent_set+",
                        to="contenttypes.ContentType",
                    ),
                ),
                (
                    "test",
                    models.CharField(max_length=255, default="test_non_polymorphic_parent"),
                ),
            ],
            options={
                "abstract": False,
                "base_manager_name": "objects",
            },
            bases=("auth.group", models.Model),
        ),
    ]
